#ifndef POTENTIAL_H
#define POTENTIAL_H

#include "rumd/KernelPlan.h"
#include <string>
#include "rumd/RUMD_Error.h"

class Sample; class ParticleData; class SimulationBox;

class Potential{
 private:
  Potential(const Potential&); 
  Potential& operator=(const Potential&); 
  std::string ID_String;
  
 protected:
  KernelPlan kp;
  ParticleData* particleData;
  SimulationBox* simBox;

  bool verbose;
  
  unsigned int nParticles;
  
  unsigned* h_Type;

  float4* d_r;
  float4* d_f;
  float4* d_w; 
  float4* d_sts;
  float4* d_misc; 
   
 public:
  Potential();
  virtual ~Potential(){}

  virtual Potential* Copy(){ throw RUMD_Error("Potential","Copy","Not implemented yet"); }  
  virtual void CalcF(bool initialize, bool calc_stresses) = 0;
  
  // Set methods.
  virtual void SetSampleParameters(KernelPlan& kPlan, ParticleData* pd, SimulationBox* simBox, float4* d_misc );
  virtual void SetVerbose(bool vb) { verbose = vb; }
  void SetID_String(const std::string& set_ID_String){ ID_String = set_ID_String; }
  
  // Get methods.
  virtual double GetPotentialEnergy() { return 0.; }
  std::string GetID_String() const { return ID_String; }
};

#endif // POTENTIAL_H
